"""Unearthed Challenge model."""
import json
import logging
from uuid import UUID

import requests
from unearthed.cli.auth import auth_handler
from unearthed.cli.config import LAMBDA_URL

CHALLENGE_FILENAME = ".unearthed_challenge.json"

logger = logging.getLogger(__name__)


class Challenge:
    """An Unearthed Challenge."""

    _logger = logging.getLogger(__name__)
    challengeId: UUID
    innovatorPlatformRef: UUID
    name: str
    slug: str
    link: str
    start: str
    end: str
    templateUrl: str
    publicDataUrl: str
    scoringFunctionUrl: str
    dockerImage: str
    dataVersion: str
    createdAt: str
    updatedAt: str

    def __init__(
        self,
        challengeId,
        innovatorPlatformRef,
        name,
        slug,
        link,
        start,
        end,
        templateUrl,
        publicDataUrl,
        scoringFunctionUrl,
        dockerImage,
        dataVersion,
        createdAt,
        updatedAt,
        **kwargs,
    ):
        """Construct Challenge instance from **kwargs."""
        self.challengeId = challengeId
        self.innovatorPlatformRef = innovatorPlatformRef
        self.name = name
        self.slug = slug
        self.link = link
        self.start = start
        self.end = end
        self.templateUrl = templateUrl
        self.publicDataUrl = publicDataUrl
        self.dockerImage = dockerImage
        self.scoringFunctionUrl = scoringFunctionUrl
        self.dataVersion = dataVersion
        self.createdAt = createdAt
        self.updatedAt = updatedAt

    def save(self, filename=CHALLENGE_FILENAME):
        """Serialize the Challenge to a JSON file."""
        logger.debug(f"saving to {filename}")
        with open(filename, "w") as json_file:
            json.dump(self.__dict__, json_file, indent=2)

    @staticmethod
    def fromJsonFile(filename):
        """Create a Challenge instance from a saved API response."""
        Challenge._logger.debug("fromJsonFile")
        with open(filename, "r") as f:
            challenge_dict = json.load(f)
            return Challenge(**challenge_dict)

    @staticmethod
    def fromApiById(id):
        """Create a Challenge instance from the Crowd ML API by Id."""
        Challenge._logger.debug("fromApiById")
        url = LAMBDA_URL + "challenges/" + id
        response = requests.get(url, auth=auth_handler, timeout=20)
        response.raise_for_status()
        challenge_dict = response.json()["challenge"]
        return Challenge(**challenge_dict)

    @staticmethod
    def allFromApi():
        """Create challenge instances from the Crowd ML API."""
        url = LAMBDA_URL + "challenges/"
        response = requests.get(url, auth=auth_handler, timeout=20)
        response.raise_for_status()
        if "challenges" in response.json():
            challenges = []
            for challenge_dict in response.json()["challenges"]:
                challenges.append(Challenge(**challenge_dict))
            return challenges
        else:
            return []



"""Unearthed Challenge command."""
import logging

import click
import requests
from unearthed.cli.auth import auth_handler
from unearthed.cli.config import LAMBDA_URL
from unearthed.cli.auth import authorize

from unearthed.core.models.challenge import Challenge

logger = logging.getLogger(__name__)


@click.group()
def challenge():
    """Inspect Unearthed challenges."""
    pass


@challenge.command("ls")
@authorize
def ls():
    """List challenges."""
    for challenge in Challenge.allFromApi():
        print(
            f"{challenge['slug']}: {challenge['name']} "
            f"({challenge['innovatorPlatformRef']})"
        )



"""Unearthed Reset."""
import os
import shutil

import click
from unearthed.cli.auth import authorize
from unearthed.cli.util import get_challenge, load_archive, load_public_data


@click.group()
def reset():
    """Reset commands."""
    pass


@reset.command()
@authorize
def data():
    """Reset the data directory."""
    current_challenge = get_challenge()
    challenge = get_challenge(id=current_challenge.challengeId)

    # check we are in a challenge dir!

    data_dir = os.path.join(os.getcwd(), "data")

    click.echo("Loading in data files...")
    if click.confirm(
        f"Remove all files in {data_dir} and re-download from Unearthed?",
        default=False,
        abort=True,
    ):
        shutil.rmtree("data", ignore_errors=True)

    load_public_data(challenge)
    return click.echo("The data directory has been reset.")


@reset.command()
@authorize
def scoring_function():
    """Reset the scoring function."""
    current_challenge = get_challenge()
    challenge = get_challenge(id=current_challenge.challengeId)

    # load in scoring function from archive
    click.echo("Loading in scoring function...")

    if click.confirm(
        f"Remove the score.py in {os.getcwd()} and redownload from Unearthed?",
        default=False,
        abort=True,
    ):
        shutil.rmtree("scoring_function", ignore_errors=True)
        if os.path.exists("score.py"):
            os.remove("score.py")

    load_archive(
        challenge.scoringFunctionUrl, "scoring_function", challenge.slug + "_scoring"
    )

    # move the file
    os.rename("scoring_function/score.py", "score.py")
    shutil.rmtree("scoring_function", ignore_errors=True)
    return click.echo("The scoring_function has been reset.")



"""Unearthed Sumission Entity."""
import json
import logging
from uuid import UUID

import requests
from unearthed.cli.auth import auth_handler
from unearthed.cli.config import LAMBDA_URL

_LAST_SUBMISSION_FILENAME = '.unearthed_last_submission.json'


class Submission():
    """A submission to an Unearthed challenge."""

    _logger = logging.getLogger(__name__)
    submissionId: UUID
    challengeRef: UUID
    userRef: UUID
    teamRef: UUID
    status: str
    createdAt: str
    updatedAt: str
    sourceUploadUrl: str
    configUploadUrl: str
    innovatorPortalSubmissionRef: UUID

    def __init__(self,
                 submissionId,
                 challengeRef,
                 userRef,
                 teamRef,
                 status,
                 createdAt,
                 updatedAt,
                 sourceUploadUrl,
                 configUploadUrl,
                 innovatorPortalSubmissionRef):
        self.submissionId = submissionId
        self.challengeRef = challengeRef
        self.userRef = userRef
        self.teamRef = teamRef
        self.status = status
        self.createdAt = createdAt
        self.updatedAt = updatedAt
        self.sourceUploadUrl = sourceUploadUrl
        self.configUploadUrl = configUploadUrl
        self.innovatorPortalSubmissionRef = innovatorPortalSubmissionRef

    @staticmethod
    def fromJsonFile(filename):
        """Create a Submission isntance from a saved API response."""
        with open(filename, 'r') as f:
            submission_dict = json.load(f)
            return Submission(**submission_dict)

    @staticmethod
    def fromApi(challengeId):
        """Create a Submission instance from the Crowd ML API."""
        Submission._logger.debug('fromApi')
        url = LAMBDA_URL + f'challenges/{challengeId}/submission'
        Submission._logger.debug(f'posting to {url}')
        response = requests.post(url=url, auth=auth_handler, timeout=10)
        response.raise_for_status()
        submission_dict = response.json()['submission']
        submission_dict['sourceUploadUrl'] = response.json()['sourceUploadUrl']
        submission_dict['innovatorPortalSubmissionRef'] = response.json()['innovatorPortalSubmissionRef']
        submission_dict['configUploadUrl'] = response.json()['configUploadUrl']
        submission = Submission(**submission_dict)
        Submission._save(submission_dict)
        return submission

    @staticmethod
    def _save(submission_dict):
        """Save the Submission to disk."""
        with open(_LAST_SUBMISSION_FILENAME, 'w') as json_file:
            json.dump(submission_dict, json_file, indent=2)
